@@ -208,7 +208,7 @@ static void dns_child_callback (const rfc1035_header_t *dns)
 	pthread_mutex_unlock (&opcode_mutex);
 }
 
-static void *dns_child_loop (__attribute__((unused)) void *dummy)
+static int dns_run_pcap_loop (void)
 {
 	pcap_t *pcap_obj;
 	char    pcap_error[PCAP_ERRBUF_SIZE];
@@ -228,7 +228,7 @@ static void *dns_child_loop (__attribute__((unused)) void *dummy)
 	pcap_obj = pcap_open_live ((pcap_device != NULL) ? pcap_device : "any",
 			PCAP_SNAPLEN,
 			0 /* Not promiscuous */,
-			(int) CDTIME_T_TO_MS (interval_g / 2),
+			(int) CDTIME_T_TO_MS (plugin_get_interval () / 2),
 			pcap_error);
 	if (pcap_obj == NULL)
 	{
@@ -236,19 +236,24 @@ static void *dns_child_loop (__attribute__((unused)) void *dummy)
 				"failed: %s",
 				(pcap_device != NULL) ? pcap_device : "any",
 				pcap_error);
-		return (NULL);
+		return (PCAP_ERROR);
 	}
 
 	memset (&fp, 0, sizeof (fp));
-	if (pcap_compile (pcap_obj, &fp, "udp port 53", 1, 0) < 0)
+	status = pcap_compile (pcap_obj, &fp, "udp port 53", 1, 0);
+	if (status < 0)
 	{
-		ERROR ("dns plugin: pcap_compile failed");
-		return (NULL);
+		ERROR ("dns plugin: pcap_compile failed: %s",
+				pcap_statustostr (status));
+		return (status);
 	}
-	if (pcap_setfilter (pcap_obj, &fp) < 0)
+
+	status = pcap_setfilter (pcap_obj, &fp);
+	if (status < 0)
 	{
-		ERROR ("dns plugin: pcap_setfilter failed");
-		return (NULL);
+		ERROR ("dns plugin: pcap_setfilter failed: %s",
+				pcap_statustostr (status));
+		return (status);
 	}
 
 	DEBUG ("dns plugin: PCAP object created.");
@@ -259,19 +264,65 @@ static void *dns_child_loop (__attribute__((unused)) void *dummy)
 	status = pcap_loop (pcap_obj,
 			-1 /* loop forever */,
 			handle_pcap /* callback */,
-			NULL /* Whatever this means.. */);
-	if (status < 0)
-		ERROR ("dns plugin: Listener thread is exiting "
-				"abnormally: %s", pcap_geterr (pcap_obj));
-
-	DEBUG ("dns plugin: Child is exiting.");
+			NULL /* user data */);
+	INFO ("dns plugin: pcap_loop exited with status %i.", status);
+	/* We need to handle "PCAP_ERROR" specially because libpcap currently
+	 * doesn't return PCAP_ERROR_IFACE_NOT_UP for compatibility reasons. */
+	if (status == PCAP_ERROR)
+		status = PCAP_ERROR_IFACE_NOT_UP;
 
 	pcap_close (pcap_obj);
-	listen_thread_init = 0;
-	pthread_exit (NULL);
+	return (status);
+} /* int dns_run_pcap_loop */
 
+static int dns_sleep_one_interval (void) /* {{{ */
+{
+	cdtime_t interval;
+	struct timespec ts = { 0, 0 };
+	int status = 0;
+
+	interval = plugin_get_interval ();
+	CDTIME_T_TO_TIMESPEC (interval, &ts);
+
+	while (42)
+	{
+		struct timespec rem = { 0, 0 };
+
+		status = nanosleep (&ts, &rem);
+		if (status == 0)
+			break;
+		else if ((errno == EINTR) || (errno == EAGAIN))
+		{
+			ts = rem;
+			continue;
+		}
+		else
+			break;
+	}
+
+	return (status);
+} /* }}} int dns_sleep_one_interval */
+
+static void *dns_child_loop (__attribute__((unused)) void *dummy) /* {{{ */
+{
+	int status;
+
+	while (42)
+	{
+		status = dns_run_pcap_loop ();
+		if (status != PCAP_ERROR_IFACE_NOT_UP)
+			break;
+
+		dns_sleep_one_interval ();
+	}
+
+	if (status != PCAP_ERROR_BREAK)
+		ERROR ("dns plugin: PCAP returned error %s.",
+				pcap_statustostr (status));
+
+	listen_thread_init = 0;
 	return (NULL);
-} /* static void dns_child_loop (void) */
+} /* }}} void *dns_child_loop */
 
 static int dns_init (void)
 {
@@ -286,7 +337,7 @@ static int dns_init (void)
 	if (listen_thread_init != 0)
 		return (-1);
 
-	status = pthread_create (&listen_thread, NULL, dns_child_loop,
+	status = plugin_thread_create (&listen_thread, NULL, dns_child_loop,
 			(void *) 0);
 	if (status != 0)
 	{
