@@ -1,6 +1,7 @@
 /**
  * collectd - src/irq.c
  * Copyright (C) 2007  Peter Holik
+ * Copyright (C) 2011  Florian Forster
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -24,6 +25,7 @@
 #include "common.h"
 #include "plugin.h"
 #include "configfile.h"
+#include "utils_ignorelist.h"
 
 #if !KERNEL_LINUX
 # error "No applicable input method."
@@ -39,82 +41,44 @@ static const char *config_keys[] =
 };
 static int config_keys_num = STATIC_ARRAY_SIZE (config_keys);
 
-static char         **irq_list;
-static unsigned int   irq_list_num = 0;
+static ignorelist_t *ignorelist = NULL;
 
-/* 
- * irq_list_action:
- * 0 => default is to collect selected irqs
- * 1 => ignore selcted irqs
+/*
+ * Private functions
  */
-static int irq_list_action;
-
 static int irq_config (const char *key, const char *value)
 {
+	if (ignorelist == NULL)
+		ignorelist = ignorelist_create (/* invert = */ 1);
+
 	if (strcasecmp (key, "Irq") == 0)
 	{
-		char **temp;
-
-		temp = realloc (irq_list, (irq_list_num + 1) * sizeof (*irq_list));
-		if (temp == NULL)
-		{
-			fprintf (stderr, "irq plugin: Cannot allocate more memory.\n");
-			ERROR ("irq plugin: Cannot allocate more memory.");
-			return (1);
-		}
-		irq_list = temp;
-
-		irq_list[irq_list_num] = strdup (value);
-		if (irq_list[irq_list_num] == NULL)
-		{
-			ERROR ("irq plugin: strdup(3) failed.");
-			return (1);
-		}
-
-		irq_list_num++;
+		ignorelist_add (ignorelist, value);
 	}
 	else if (strcasecmp (key, "IgnoreSelected") == 0)
 	{
+		int invert = 1;
 		if (IS_TRUE (value))
-			irq_list_action = 1;
-		else
-			irq_list_action = 0;
+			invert = 0;
+		ignorelist_set_invert (ignorelist, invert);
 	}
 	else
 	{
 		return (-1);
 	}
+
 	return (0);
 }
 
-/*
- * Check if this interface/instance should be ignored. This is called from
- * both, `submit' and `write' to give client and server the ability to
- * ignore certain stuff..
- */
-static int check_ignore_irq (const char *irq)
-{
-	unsigned int i;
-
-	if (irq_list_num < 1)
-		return (0);
-
-	for (i = 0; i < irq_list_num; i++)
-		if (strcmp (irq, irq_list[i]) == 0)
-			return (irq_list_action);
-
-	return (1 - irq_list_action);
-}
-
-static void irq_submit (const char *irq_name, counter_t value)
+static void irq_submit (const char *irq_name, derive_t value)
 {
 	value_t values[1];
 	value_list_t vl = VALUE_LIST_INIT;
 
-	if (check_ignore_irq (irq_name))
+	if (ignorelist_match (ignorelist, irq_name) != 0)
 		return;
 
-	values[0].counter = value;
+	values[0].derive = value;
 
 	vl.values = values;
 	vl.values_len = 1;
@@ -130,30 +94,57 @@ static int irq_read (void)
 {
 	FILE *fh;
 	char buffer[1024];
-	unsigned long long irq_value;
-	unsigned long long value;
-	char *endptr;
-	int i;
+	int  cpu_count;
+	char *fields[256];
 
-	char *fields[64];
-	int fields_num;
-
-	if ((fh = fopen ("/proc/interrupts", "r")) == NULL)
+	/*
+	 * Example content:
+	 *         CPU0       CPU1       CPU2       CPU3
+	 * 0:       2574          1          3          2   IO-APIC-edge      timer
+	 * 1:     102553     158669     218062      70587   IO-APIC-edge      i8042
+	 * 8:          0          0          0          1   IO-APIC-edge      rtc0
+	 */
+	fh = fopen ("/proc/interrupts", "r");
+	if (fh == NULL)
 	{
 		char errbuf[1024];
-		WARNING ("irq plugin: fopen (/proc/interrupts): %s",
+		ERROR ("irq plugin: fopen (/proc/interrupts): %s",
 				sstrerror (errno, errbuf, sizeof (errbuf)));
 		return (-1);
 	}
+
+	/* Get CPU count from the first line */
+	if(fgets (buffer, sizeof (buffer), fh) != NULL) {
+		cpu_count = strsplit (buffer, fields,
+				STATIC_ARRAY_SIZE (fields));
+	} else {
+		ERROR ("irq plugin: unable to get CPU count from first line "
+				"of /proc/interrupts");
+		return (-1);
+	}
+
 	while (fgets (buffer, sizeof (buffer), fh) != NULL)
 	{
 		char *irq_name;
 		size_t irq_name_len;
+		derive_t irq_value;
+		int i;
+		int fields_num;
+		int irq_values_to_parse;
 
-		fields_num = strsplit (buffer, fields, 64);
+		fields_num = strsplit (buffer, fields,
+				STATIC_ARRAY_SIZE (fields));
 		if (fields_num < 2)
 			continue;
 
+		/* Parse this many numeric fields, skip the rest
+		 * (+1 because first there is a name of irq in each line) */
+		if (fields_num >= cpu_count + 1)
+			irq_values_to_parse = cpu_count;
+		else
+			irq_values_to_parse = fields_num - 1;
+
+		/* First field is irq name and colon */
 		irq_name = fields[0];
 		irq_name_len = strlen (irq_name);
 		if (irq_name_len < 2)
@@ -168,25 +159,24 @@ static int irq_read (void)
 		irq_name_len--;
 
 		irq_value = 0;
-		for (i = 1; i < fields_num; i++)
+		for (i = 1; i <= irq_values_to_parse; i++)
 		{
-			errno = 0;
-			endptr = NULL;
-			value = strtoull (fields[i], &endptr, 10);
+			/* Per-CPU value */
+			value_t v;
+			int status;
 
-			/* Ignore all fields following a non-numeric field. */
-			if ((errno != 0) || (endptr == NULL) || (*endptr != 0))
+			status = parse_value (fields[i], &v, DS_TYPE_DERIVE);
+			if (status != 0)
 				break;
 
-			irq_value += value;
+			irq_value += v.derive;
 		} /* for (i) */
 
 		/* No valid fields -> do not submit anything. */
 		if (i <= 1)
 			continue;
 
-		/* Force 32bit wrap-around */
-		irq_submit (irq_name, irq_value % 4294967296ULL);
+		irq_submit (irq_name, irq_value);
 	}
 
 	fclose (fh);
