@@ -1,6 +1,7 @@
 /**
  * collectd - src/mbmon.c
- * Copyright (C) 2006 Flavio Stanchina
+ * Copyright (C) 2006       Flavio Stanchina
+ * Copyright (C) 2006-2007  Florian octo Forster
  * Based on the hddtemp plugin.
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -19,6 +20,7 @@
  *
  * Authors:
  *   Flavio Stanchina <flavio at stanchina.net>
+ *   Florian Forster <octo at verplant.org>
  **/
 
 #include "collectd.h"
@@ -227,12 +229,12 @@ static void mbmon_submit (const char *type, const char *type_instance,
 
 	vl.values = values;
 	vl.values_len = 1;
-	vl.time = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "mbmon", sizeof (vl.plugin));
-	strncpy (vl.type_instance, type_instance, sizeof (vl.type_instance));
+	sstrncpy (vl.type, type, sizeof (vl.type));
+	sstrncpy (vl.type_instance, type_instance, sizeof (vl.type_instance));
 
-	plugin_dispatch_values (type, &vl);
+	plugin_dispatch_values (&vl);
 } /* void mbmon_submit */
 
 /* Trim trailing whitespace from a string. */
