@@ -7,6 +7,7 @@
  * Copyright (C) 2009       Andrés J. Díaz
  * Copyright (C) 2009       Manuel Sanmartin
  * Copyright (C) 2010       Clément Stenac
+ * Copyright (C) 2012       Cosmin Ioiart
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -30,6 +31,7 @@
  *   Andrés J. Díaz <ajdiaz at connectical.com>
  *   Manuel Sanmartin
  *   Clément Stenac <clement.stenac at diwi.org>
+ *   Cosmin Ioiart <cioiart at gmail.com>
  **/
 
 #include "collectd.h"
@@ -109,6 +111,11 @@
 #define MAXARGLN 1024
 /* #endif HAVE_PROCINFO_H */
 
+#elif KERNEL_SOLARIS
+# include <procfs.h>
+# include <dirent.h>
+/* #endif KERNEL_SOLARIS */
+
 #else
 # error "No applicable input method."
 #endif
@@ -117,6 +124,10 @@
 # include <regex.h>
 #endif
 
+#if HAVE_KSTAT_H
+# include <kstat.h>
+#endif
+
 #ifndef ARG_MAX
 #  define ARG_MAX 4096
 #endif
@@ -216,8 +227,8 @@ int getargs (struct procentry64 *processBuffer, int bufferLen, char *argsBuffer,
 #endif /* HAVE_PROCINFO_H */
 
 /* put name of process from config to list_head_g tree
-   list_head_g is a list of 'procstat_t' structs with
-   processes names we want to watch */
+ * list_head_g is a list of 'procstat_t' structs with
+ * processes names we want to watch */
 static void ps_list_register (const char *name, const char *regexp)
 {
 	procstat_t *new;
@@ -712,7 +723,7 @@ static void ps_submit_proc_list (procstat_t *ps)
 	}
 
 	DEBUG ("name = %s; num_proc = %lu; num_lwp = %lu; "
-                        "vmem_size = %lu; vmem_rss = %lu; vmem_data = %lu; "
+			"vmem_size = %lu; vmem_rss = %lu; vmem_data = %lu; "
 			"vmem_code = %lu; "
 			"vmem_minflt_counter = %"PRIi64"; vmem_majflt_counter = %"PRIi64"; "
 			"cpu_user_counter = %"PRIi64"; cpu_system_counter = %"PRIi64"; "
@@ -726,6 +737,26 @@ static void ps_submit_proc_list (procstat_t *ps)
 			ps->io_rchar, ps->io_wchar, ps->io_syscr, ps->io_syscw);
 } /* void ps_submit_proc_list */
 
+#if KERNEL_LINUX || KERNEL_SOLARIS
+static void ps_submit_fork_rate (derive_t value)
+{
+	value_t values[1];
+	value_list_t vl = VALUE_LIST_INIT;
+
+	values[0].derive = value;
+
+	vl.values = values;
+	vl.values_len = 1;
+	sstrncpy(vl.host, hostname_g, sizeof (vl.host));
+	sstrncpy(vl.plugin, "processes", sizeof (vl.plugin));
+	sstrncpy(vl.plugin_instance, "", sizeof (vl.plugin_instance));
+	sstrncpy(vl.type, "fork_rate", sizeof (vl.type));
+	sstrncpy(vl.type_instance, "", sizeof (vl.type_instance));
+
+	plugin_dispatch_values(&vl);
+}
+#endif /* KERNEL_LINUX || KERNEL_SOLARIS*/
+
 /* ------- additional functions for KERNEL_LINUX/HAVE_THREAD_INFO ------- */
 #if KERNEL_LINUX
 static int ps_read_tasks (int pid)
@@ -780,7 +811,7 @@ static procstat_t *ps_read_vmem (int pid, procstat_t *ps)
 			continue;
 
 		numfields = strsplit (buffer, fields,
-                                      STATIC_ARRAY_SIZE (fields));
+				STATIC_ARRAY_SIZE (fields));
 
 		if (numfields < 2)
 			continue;
@@ -1117,70 +1148,226 @@ static char *ps_get_cmdline (pid_t pid, char *name, char *buf, size_t buf_len)
 	return buf;
 } /* char *ps_get_cmdline (...) */
 
-static unsigned long read_fork_rate ()
+static int read_fork_rate ()
 {
 	FILE *proc_stat;
-	char buf[1024];
-	unsigned long result = 0;
-	int numfields;
-	char *fields[3];
+	char buffer[1024];
+	value_t value;
+	_Bool value_valid = 0;
 
-	proc_stat = fopen("/proc/stat", "r");
-	if (proc_stat == NULL) {
+	proc_stat = fopen ("/proc/stat", "r");
+	if (proc_stat == NULL)
+	{
 		char errbuf[1024];
 		ERROR ("processes plugin: fopen (/proc/stat) failed: %s",
 				sstrerror (errno, errbuf, sizeof (errbuf)));
-		return ULONG_MAX;
+		return (-1);
 	}
 
-	while (fgets (buf, sizeof(buf), proc_stat) != NULL)
+	while (fgets (buffer, sizeof (buffer), proc_stat) != NULL)
 	{
-		char *endptr;
+		int status;
+		char *fields[3];
+		int fields_num;
 
-		numfields = strsplit(buf, fields, STATIC_ARRAY_SIZE (fields));
-		if (numfields != 2)
+		fields_num = strsplit (buffer, fields,
+				STATIC_ARRAY_SIZE (fields));
+		if (fields_num != 2)
 			continue;
 
 		if (strcmp ("processes", fields[0]) != 0)
 			continue;
 
-		errno = 0;
-		endptr = NULL;
-		result = strtoul(fields[1], &endptr, /* base = */ 10);
-		if ((endptr == fields[1]) || (errno != 0)) {
-			ERROR ("processes plugin: Cannot parse fork rate: %s",
-					fields[1]);
-			result = ULONG_MAX;
-			break;
-		}
+		status = parse_value (fields[1], &value, DS_TYPE_DERIVE);
+		if (status == 0)
+			value_valid = 1;
 
 		break;
 	}
-
 	fclose(proc_stat);
 
-	return result;
-}
+	if (!value_valid)
+		return (-1);
 
-static void ps_submit_fork_rate (unsigned long value)
+	ps_submit_fork_rate (value.derive);
+	return (0);
+}
+#endif /*KERNEL_LINUX */
+
+#if KERNEL_SOLARIS
+static const char *ps_get_cmdline (pid_t pid, /* {{{ */
+		char *buffer, size_t buffer_size)
 {
-	value_t values[1];
-	value_list_t vl = VALUE_LIST_INIT;
+	char path[PATH_MAX];
+	psinfo_t info;
+	int status;
 
-	values[0].derive = (derive_t) value;
+	snprintf(path, sizeof (path), "/proc/%i/psinfo", pid);
 
-	vl.values = values;
-	vl.values_len = 1;
-	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
-	sstrncpy (vl.plugin, "processes", sizeof (vl.plugin));
-	sstrncpy (vl.plugin_instance, "", sizeof (vl.plugin_instance));
-	sstrncpy (vl.type, "fork_rate", sizeof (vl.type));
-	sstrncpy (vl.type_instance, "", sizeof (vl.type_instance));
+	status = read_file_contents (path, (void *) &info, sizeof (info));
+	if (status != ((int) buffer_size))
+	{
+		ERROR ("processes plugin: Unexpected return value "
+				"while reading \"%s\": "
+				"Returned %i but expected %zu.",
+				path, status, buffer_size);
+		return (NULL);
+	}
 
-	plugin_dispatch_values (&vl);
+	info.pr_psargs[sizeof (info.pr_psargs) - 1] = 0;
+	sstrncpy (buffer, info.pr_psargs, buffer_size);
+
+	return (buffer);
+} /* }}} int ps_get_cmdline */
+
+/*
+ * Reads process information on the Solaris OS. The information comes mainly from
+ * /proc/PID/status, /proc/PID/psinfo and /proc/PID/usage
+ * The values for input and ouput chars are calculated "by hand"
+ * Added a few "solaris" specific process states as well
+ */
+static int ps_read_process(int pid, procstat_t *ps, char *state)
+{
+	char filename[64];
+	char f_psinfo[64], f_usage[64];
+	char *buffer;
+
+	pstatus_t *myStatus;
+	psinfo_t *myInfo;
+	prusage_t *myUsage;
+
+	snprintf(filename, sizeof (filename), "/proc/%i/status", pid);
+	snprintf(f_psinfo, sizeof (f_psinfo), "/proc/%i/psinfo", pid);
+	snprintf(f_usage, sizeof (f_usage), "/proc/%i/usage", pid);
+
+
+	buffer = malloc(sizeof (pstatus_t));
+	memset(buffer, 0, sizeof (pstatus_t));
+	read_file_contents(filename, buffer, sizeof (pstatus_t));
+	myStatus = (pstatus_t *) buffer;
+
+	buffer = malloc(sizeof (psinfo_t));
+	memset(buffer, 0, sizeof(psinfo_t));
+	read_file_contents(f_psinfo, buffer, sizeof (psinfo_t));
+	myInfo = (psinfo_t *) buffer;
+
+	buffer = malloc(sizeof (prusage_t));
+	memset(buffer, 0, sizeof(prusage_t));
+	read_file_contents(f_usage, buffer, sizeof (prusage_t));
+	myUsage = (prusage_t *) buffer;
+
+	sstrncpy(ps->name, myInfo->pr_fname, sizeof (myInfo->pr_fname));
+	ps->num_lwp = myStatus->pr_nlwp;
+	if (myInfo->pr_wstat != 0) {
+		ps->num_proc = 0;
+		ps->num_lwp = 0;
+		*state = (char) 'Z';
+		return (0);
+	} else {
+		ps->num_proc = 1;
+		ps->num_lwp = myInfo->pr_nlwp;
+	}
+
+	/*
+	 * Convert system time and user time from nanoseconds to microseconds
+	 * for compatibility with the linux module
+	 */
+	ps->cpu_system_counter = myStatus -> pr_stime.tv_nsec / 1000;
+	ps->cpu_user_counter = myStatus -> pr_utime.tv_nsec / 1000;
+
+	/*
+	 * Convert rssize from KB to bytes to be consistent w/ the linux module
+	 */
+	ps->vmem_rss = myInfo->pr_rssize * 1024;
+	ps->vmem_size = myInfo->pr_size * 1024;
+	ps->vmem_minflt_counter = myUsage->pr_minf;
+	ps->vmem_majflt_counter = myUsage->pr_majf;
+
+	/*
+	 * TODO: Data and code segment calculations for Solaris
+	 */
+
+	ps->vmem_data = -1;
+	ps->vmem_code = -1;
+	ps->stack_size = myStatus->pr_stksize;
+
+	/*
+	 * Calculating input/ouput chars
+	 * Formula used is total chars / total blocks => chars/block
+	 * then convert input/output blocks to chars
+	 */
+	ulong_t tot_chars = myUsage->pr_ioch;
+	ulong_t tot_blocks = myUsage->pr_inblk + myUsage->pr_oublk;
+	ulong_t chars_per_block = 1;
+	if (tot_blocks != 0)
+		chars_per_block = tot_chars / tot_blocks;
+	ps->io_rchar = myUsage->pr_inblk * chars_per_block;
+	ps->io_wchar = myUsage->pr_oublk * chars_per_block;
+	ps->io_syscr = myUsage->pr_sysc;
+	ps->io_syscw = myUsage->pr_sysc;
+
+
+	/*
+	 * TODO: Find way of setting BLOCKED and PAGING status
+	 */
+
+	*state = (char) 'R';
+	if (myStatus->pr_flags & PR_ASLEEP)
+		*state = (char) 'S';
+	else if (myStatus->pr_flags & PR_STOPPED)
+		*state = (char) 'T';
+	else if (myStatus->pr_flags & PR_DETACH)
+		*state = (char) 'E';
+	else if (myStatus->pr_flags & PR_DAEMON)
+		*state = (char) 'A';
+	else if (myStatus->pr_flags & PR_ISSYS)
+		*state = (char) 'Y';
+	else if (myStatus->pr_flags & PR_ORPHAN)
+		*state = (char) 'O';
+
+	sfree(myStatus);
+	sfree(myInfo);
+	sfree(myUsage);
+
+	return (0);
 }
 
-#endif /* KERNEL_LINUX */
+/*
+ * Reads the number of threads created since the last reboot. On Solaris these
+ * are retrieved from kstat (module cpu, name sys, class misc, stat nthreads).
+ * The result is the sum for all the threads created on each cpu
+ */
+static int read_fork_rate()
+{
+	extern kstat_ctl_t *kc;
+	kstat_t *ksp_chain = NULL;
+	derive_t result = 0;
+
+	if (kc == NULL)
+		return (-1);
+
+	for (ksp_chain = kc->kc_chain;
+			ksp_chain != NULL;
+			ksp_chain = ksp_chain->ks_next)
+	{
+		if ((strcmp (ksp_chain->ks_module, "cpu") == 0)
+				&& (strcmp (ksp_chain->ks_name, "sys") == 0)
+				&& (strcmp (ksp_chain->ks_class, "misc") == 0))
+		{
+			long long tmp;
+
+			kstat_read (kc, ksp_chain, NULL);
+
+			tmp = get_kstat_value(ksp_chain, "nthreads");
+			if (tmp != -1LL)
+				result += tmp;
+		}
+	}
+
+	ps_submit_fork_rate (result);
+	return (0);
+}
+#endif /* KERNEL_SOLARIS */
 
 #if HAVE_THREAD_INFO
 static int mach_get_task_name (task_t t, int *pid, char *name, size_t name_max_len)
@@ -1506,8 +1693,6 @@ static int ps_read (void)
 	procstat_entry_t pse;
 	char       state;
 
-	unsigned long fork_rate;
-
 	procstat_t *ps_ptr;
 
 	running = sleeping = zombies = stopped = paging = blocked = 0;
@@ -1589,9 +1774,7 @@ static int ps_read (void)
 	for (ps_ptr = list_head_g; ps_ptr != NULL; ps_ptr = ps_ptr->next)
 		ps_submit_proc_list (ps_ptr);
 
-	fork_rate = read_fork_rate();
-	if (fork_rate != ULONG_MAX)
-		ps_submit_fork_rate(fork_rate);
+	read_fork_rate();
 /* #endif KERNEL_LINUX */
 
 #elif HAVE_LIBKVM_GETPROCS && HAVE_STRUCT_KINFO_PROC_FREEBSD
@@ -1605,9 +1788,9 @@ static int ps_read (void)
 
 	kvm_t *kd;
 	char errbuf[1024];
-  	struct kinfo_proc *procs;          /* array of processes */
+	struct kinfo_proc *procs;          /* array of processes */
 	struct kinfo_proc *proc_ptr = NULL;
-  	int count;                         /* returns number of processes */
+	int count;                         /* returns number of processes */
 	int i;
 
 	procstat_t *ps_ptr;
@@ -1778,7 +1961,7 @@ static int ps_read (void)
 				if (procentry[i].pi_pid == 0)
 					cmdline = "swapper";
 				cargs = cmdline;
- 			}
+			}
 			else
 			{
 				if (getargs(&procentry[i], sizeof(struct procentry64), arglist, MAXARGLN) >= 0)
@@ -1870,7 +2053,118 @@ static int ps_read (void)
 
 	for (ps = list_head_g; ps != NULL; ps = ps->next)
 		ps_submit_proc_list (ps);
-#endif /* HAVE_PROCINFO_H */
+/* #endif HAVE_PROCINFO_H */
+
+#elif KERNEL_SOLARIS
+	/*
+	 * The Solaris section adds a few more process states and removes some
+	 * process states compared to linux. Most notably there is no "PAGING"
+	 * and "BLOCKED" state for a process.  The rest is similar to the linux
+	 * code.
+	 */
+	int running = 0;
+	int sleeping = 0;
+	int zombies = 0;
+	int stopped = 0;
+	int detached = 0;
+	int daemon = 0;
+	int system = 0;
+	int orphan = 0;
+
+	struct dirent *ent;
+	DIR *proc;
+
+	int status;
+	procstat_t *ps_ptr;
+	char state;
+
+	char cmdline[PRARGSZ];
+
+	ps_list_reset ();
+
+	proc = opendir ("/proc");
+	if (proc == NULL)
+		return (-1);
+
+	while ((ent = readdir(proc)) != NULL)
+	{
+		int pid;
+		struct procstat ps;
+		procstat_entry_t pse;
+
+		if (!isdigit ((int) ent->d_name[0]))
+			continue;
+
+		if ((pid = atoi (ent->d_name)) < 1)
+			continue;
+
+		status = ps_read_process (pid, &ps, &state);
+		if (status != 0)
+		{
+			DEBUG("ps_read_process failed: %i", status);
+			continue;
+		}
+
+		pse.id = pid;
+		pse.age = 0;
+
+		pse.num_proc   = ps.num_proc;
+		pse.num_lwp    = ps.num_lwp;
+		pse.vmem_size  = ps.vmem_size;
+		pse.vmem_rss   = ps.vmem_rss;
+		pse.vmem_data  = ps.vmem_data;
+		pse.vmem_code  = ps.vmem_code;
+		pse.stack_size = ps.stack_size;
+
+		pse.vmem_minflt = 0;
+		pse.vmem_minflt_counter = ps.vmem_minflt_counter;
+		pse.vmem_majflt = 0;
+		pse.vmem_majflt_counter = ps.vmem_majflt_counter;
+
+		pse.cpu_user = 0;
+		pse.cpu_user_counter = ps.cpu_user_counter;
+		pse.cpu_system = 0;
+		pse.cpu_system_counter = ps.cpu_system_counter;
+
+		pse.io_rchar = ps.io_rchar;
+		pse.io_wchar = ps.io_wchar;
+		pse.io_syscr = ps.io_syscr;
+		pse.io_syscw = ps.io_syscw;
+
+		switch (state)
+		{
+			case 'R': running++;  break;
+			case 'S': sleeping++; break;
+			case 'E': detached++; break;
+			case 'Z': zombies++;  break;
+			case 'T': stopped++;  break;
+			case 'A': daemon++;   break;
+			case 'Y': system++;   break;
+			case 'O': orphan++;   break;
+		}
+
+
+		ps_list_add (ps.name,
+				ps_get_cmdline ((pid_t) pid,
+					cmdline, sizeof (cmdline)),
+				&pse);
+	} /* while(readdir) */
+	closedir (proc);
+
+	ps_submit_state ("running",  running);
+	ps_submit_state ("sleeping", sleeping);
+	ps_submit_state ("zombies",  zombies);
+	ps_submit_state ("stopped",  stopped);
+	ps_submit_state ("detached", detached);
+	ps_submit_state ("daemon",   daemon);
+	ps_submit_state ("system",   system);
+	ps_submit_state ("orphan",   orphan);
+
+	for (ps_ptr = list_head_g; ps_ptr != NULL; ps_ptr = ps_ptr->next)
+		ps_submit_proc_list (ps_ptr);
+
+	read_fork_rate();
+#endif /* KERNEL_SOLARIS */
 
 	return (0);
 } /* int ps_read */
