@@ -22,7 +22,9 @@
 #ifndef UTILS_CMD_PUTNOTIF_H
 #define UTILS_CMD_PUTNOTIF_H 1
 
-int handle_putnotif (FILE *fh, char **fields, int fields_num);
+#include <stdio.h>
+
+int handle_putnotif (FILE *fh, char *buffer);
 
 /* vim: set shiftwidth=2 softtabstop=2 tabstop=8 : */
 
