@@ -260,7 +260,7 @@ uuidcache_init(void)
 			* (This is useful, if the cdrom on /dev/hdc must not
 			* be accessed.)
 			*/
-				snprintf(device, sizeof(device), "%s/%s",
+				ssnprintf(device, sizeof(device), "%s/%s",
 					DEVLABELDIR, ptname);
 				if(!get_label_uuid(device, &label, uuid)) {
 					uuidcache_addentry(sstrdup(device),
@@ -659,11 +659,11 @@ cu_mount_t *cu_mount_getlist(cu_mount_t **list)
 #elif HAVE_TWO_GETMNTENT || HAVE_GEN_GETMNTENT || HAVE_SUN_GETMNTENT
 	new = cu_mount_gen_getmntent ();
 #elif HAVE_SEQ_GETMNTENT
-# warn "This version of `getmntent' hat not yet been implemented!"
+# error "This version of `getmntent' hat not yet been implemented!"
 #elif HAVE_ONE_GETMNTENT
 	new = cu_mount_getmntent ();
 #else
-	new = NULL;
+# error "Could not determine how to find mountpoints."
 #endif
 
 	if (first != NULL)
@@ -769,8 +769,6 @@ cu_mount_getoptionvalue(char *line, char *keyword)
 	return r;
 } /* char *cu_mount_getoptionvalue(char *line, char *keyword) */
 
-
-
 int
 cu_mount_type(const char *type)
 {
@@ -782,5 +780,3 @@ cu_mount_type(const char *type)
 	return CUMT_UNKNOWN;
 } /* int cu_mount_type(const char *type) */
 
-
-
