@@ -36,11 +36,11 @@ static void cxmms_submit (const char *type, gauge_t value)
 
 	vl.values = values;
 	vl.values_len = 1;
-	vl.time = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "xmms", sizeof (vl.plugin));
+	sstrncpy (vl.type, type, sizeof (vl.type));
 
-	plugin_dispatch_values (type, &vl);
+	plugin_dispatch_values (&vl);
 } /* void cxmms_submit */
 
 int cxmms_read (void)
