@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-DEFAULT_VERSION="5.1.0.git"
+DEFAULT_VERSION="5.4.0.git"
 
 VERSION="`git describe 2> /dev/null | sed -e 's/^collectd-//'`"
 
@@ -10,8 +10,4 @@ fi
 
 VERSION="`echo \"$VERSION\" | sed -e 's/-/./g'`"
 
-if test "x`uname -s`" = "xAIX" ; then
-	echo "$VERSION\c"
-else 
-	echo -n "$VERSION"
-fi
+echo -n "$VERSION"
